import sys

from django.core.management.base import BaseCommand, CommandError
from program.models import Episode, TimeSlot
from program.utils import parse_date


class Command(BaseCommand):
    help = "updates title and content of a timeslot episode by reading both from stdin."

    def add_arguments(self, parser):
        parser.add_argument("show_id", help="ID of the show", type=int)
        parser.add_argument("date", help="date of the timeslot", type=str)
        parser.add_argument(
            "timeslot_id",
            default=None,
            help="id of the timeslot within the date.",
            nargs="?",
            type=int,
        )

    def _print_timeslot_selections(self, timeslots):
        for ts in timeslots:
            start = ts.start.strftime("%x %H:%M")
            if ts.start.date() != ts.end.date():
                end = ts.end.strftime("%x %H:%M")
            else:
                end = ts.end.strftime("%H:%M")
            self.stderr.write(f"[{ts.id}] {start} - {end}")

    def handle(self, *args, show_id, timeslot_id, **options):
        try:
            date = parse_date(options["date"])
        except ValueError:
            raise CommandError("Unable to parse the provided date (format YYYY-MM-DD).")

        timeslots = TimeSlot.objects.filter(
            schedule__show=show_id,
            start__date=date,
        ).order_by("start")

        if len(timeslots) > 1:
            if timeslot_id is None:
                self.stderr.write(
                    "More than one timeslot exists for the given show and date. "
                    "Please provide one of the given timeslot ids:"
                )
                self._print_timeslot_selections(timeslots)
                return
            timeslots = timeslots.filter(id=timeslot_id)

        timeslot = timeslots.first()

        if timeslot is None:
            raise CommandError("There is no timeslot for the given show and date.")

        try:
            title = sys.stdin.readline().strip()
            content = sys.stdin.read()
        except Exception:
            raise CommandError("Could not read title and content from stdin.")

        if episode := timeslot.episode:
            episode.title = title
            episode.content = content
            episode.save()
        else:
            timeslot.episode = Episode.objects.create(
                show_id=show_id, title=title, content=content
            )
            timeslot.save()

        self.stdout.write(self.style.SUCCESS(f'Updated episode "{title}" for {episode.timeslot}'))
